# Plugin for TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 2013 Wave Systems Corp.
# Copyright (C) 2013 Peter Thoeny, peter[at]thoeny.org 
# Copyright (C) 2013 TWiki Contributors
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, published at
# http://www.gnu.org/copyleft/gpl.html
#
# As per the GPL, removal of this notice is prohibited.

package TWiki::Plugins::WatchlistPlugin;

# =========================
our $VERSION = '$Rev: 26361 (2013-10-14) $';
our $RELEASE = '2013-09-15';
our $SHORTDESCRIPTION = 'Watch topics of interest and get notified of changes by e-mail';
our $NO_PREFS_IN_TOPIC = 1;

# =========================
my $debug = $TWiki::cfg{Plugins}{WatchlistPlugin}{Debug} || 0;
my $core;
my $baseWeb;
my $baseTopic;
my $previousAfterSave;

# =========================
sub initPlugin {
    ( $baseTopic, $baseWeb ) = @_;

    # check for Plugins.pm versions
    if( $TWiki::Plugins::VERSION < 1.2 ) {
        TWiki::Func::writeWarning( "Version mismatch between WatchlistPlugin and Plugins.pm" );
        return 0;
    }

    $core = undef;
    $previousAfterSave = '';
    TWiki::Func::registerTagHandler( 'WATCHLIST', \&_WATCHLIST );

    # Plugin correctly initialized
    TWiki::Func::writeDebug( "- WatchlistPlugin: initPlugin( "
      . "$baseWeb.$baseTopic ) is OK" ) if $debug;

    return 1;
}

# =========================
sub _WATCHLIST {
#   my ( $session, $params, $theTopic, $theWeb ) = @_;

    # Lazy loading, e.g. compile core module only when needed
    unless( $core ) {
        require TWiki::Plugins::WatchlistPlugin::Core;
        $core = new TWiki::Plugins::WatchlistPlugin::Core( $baseWeb, $baseTopic );
    }
    return $core->VarWATCHLIST( @_ );
}

# =========================
sub afterSaveHandler {
    # do not uncomment, use $_[0], $_[1]... instead
    ### my ( $text, $topic, $web, $error, $meta ) = @_;

    TWiki::Func::writeDebug( "- WatchlistPlugin: afterSaveHandler( " .
                             "$_[2].$_[1] )" ) if $debug;

    # Prevent recursion
    my $query = TWiki::Func::getCgiQuery();
    return if( $query && $query->param( 'watchlist_action' ) );
    return if( "$_[2].$_[1]" eq $previousAfterSave );
    $previousAfterSave = "$_[2].$_[1]";

    unless( $core ) {
        require TWiki::Plugins::WatchlistPlugin::Core;
        $core = new TWiki::Plugins::WatchlistPlugin::Core( $baseWeb, $baseTopic );
    }
    return $core->afterSaveHandler( @_ );
}

# =========================
sub afterRenameHandler {
    # do not uncomment, use $_[0], $_[1]... instead
    # my ( $oldWeb, $oldTopic, $oldAttachment, $newWeb, $newTopic, $newAttachment ) = @_;

    TWiki::Func::writeDebug( "- WatchlistPlugin: afterRenameHandler( " .
                             "$_[0].$_[1] $_[2] -> $_[3].$_[4] $_[5] )" ) if $debug;

    unless( $core ) {
        require TWiki::Plugins::WatchlistPlugin::Core;
        $core = new TWiki::Plugins::WatchlistPlugin::Core( $baseWeb, $baseTopic );
    }
    return $core->afterRenameHandler( @_ );
}

# =========================
# This method is designed to be invoked via the TWiki::UI::run method.
sub watchlistNotify {
    TWiki::Func::writeDebug( "- WatchlistPlugin: watchlistnotify()" ) if $debug;

    unless( $core ) {
        require TWiki::Plugins::WatchlistPlugin::Core;
        $core = new TWiki::Plugins::WatchlistPlugin::Core( $baseWeb, $baseTopic );
    }
    return $core->watchlistNotify();
}

# =========================
1;
